#include "testlib.h"

using namespace std;

typedef long long ll;

const ll MAX = (ll)1e18;


// Function to calculate (a^b) % m
long long power(long long a1, long long b, long long m) {
    __int128 result = 1;
    __int128 a = a1;
    a = a % m;
    while (b > 0) {
        if (b % 2 == 1)
            result = (result * a) % m;
        a = (a * a) % m;
        b /= 2;
    }
    return (ll) result;
}

// Function to perform Miller-Rabin primality test
bool millerRabinTest(long long n, long long d) {
    // Pick a random number 'a' in the range [2, n-2]
    long long a = 2 + rnd.next(n - 4);

    long long x = power(a, d, n);

    if (x == 1 || x == n - 1)
        return true;

    // Repeat squaring and updating x (d times)
    while (d != n - 1) {
        __int128 xx = x;
        x = (ll) ((xx * xx) % n);
        d *= 2;

        if (x == 1)
            return false;
        if (x == n - 1)
            return true;
    }

    return false;
}

// Function to check if a number is prime using the Miller-Rabin algorithm
bool isPrimeMillerRabin(long long n, int iterations = 100) {
    if (n == 1 || n == 4)
        return false;
    if (n <= 5)
        return true;

    // Find 'd' such that n-1 = 2^r * d
    long long d = n - 1;
    while (d % 2 == 0)
        d /= 2;

    // Perform Miller-Rabin test 'iterations' times
    for (int i = 0; i < iterations; ++i)
        if (!millerRabinTest(n, d))
            return false;

    return true;
}

ll pw(ll a, ll b) {
    ll r = 1;
    while (b > 0) {
        if (b % 2 == 1) {
            r *= a;
        }
        a *= a;
        b /= 2;
    }
    return r;
}

int main(int argc, char **argv)
{
    registerValidation(argc, argv);

    ll n = inf.readLong(6, MAX, "n");
    inf.readEoln();
    inf.readEof();

    ensuref(!isPrimeMillerRabin(n), "n must not be prime");

    bool ok = false;
    for (int k = 2; k <= 60; k++) {
        ll x = round(pow(n, 1.0 / k));
        if (pw(x, k) == n) {
            if (isPrimeMillerRabin(x)) {
                continue;
            }
            for (ll p = 2; p * p <= x; p++) {
                if (x % p == 0) {
                    ll q = x / p;
                    if (isPrimeMillerRabin(p) && isPrimeMillerRabin(q)) {
                        ok = true;
                        break;
                    }
                }
            }
            if (ok) {
                break;
            }
        }
    }

    if (!ok) {
        int ndiv = 0;
        for (ll p = 2; p * p * p <= n; p++) {
            while (n % p == 0) {
                ndiv++;
                n /= p;
            }
        }
        if (ndiv > 2) {
            ensuref(false, "n has not correct form");
        }
        if (!isPrimeMillerRabin(n)) {
            ndiv += 2;
        } else {
            ndiv += 1;
        }
        ensuref(ndiv == 2, "k = 1, n must have 2 prime divisors, it has %d", ndiv);
    }

    return 0;
}
